package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.BookConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonStringRequest;
import com.cultural.heritage.model.dto.book.BookingDateUpdateRequest;
import com.cultural.heritage.model.entity.BookingDate;
import com.cultural.heritage.model.vo.book.BookingDateVO;
import com.cultural.heritage.service.book.BookingDateService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/book")
@Slf4j
@Tag(name = "预约日期管理模块")
public class BookingDateController {


    @Resource
    private BookingDateService bookingDateService;


    /**
     * (写真预约)初始化未来一天的预约日期
     */
//    @Scheduled(cron = "*/5 * * * * ?")
    // 每天00:00-00:00调用
    @Scheduled(cron = "0 0 0 * * ?")
    @Transactional(rollbackFor = Exception.class)
    @PostMapping("/init")
    @Operation(summary = "(自动调用)初始化未来一天的预约日期", description = "参数：无，权限：自动调用，方法名：initCurrentBookingDate")
    public void initCurrentBookingDate() {
        // 初始化预约日期
        List<BookingDate> bookingDateList = new ArrayList<>();
        BookingDate bookingDateRent = bookingDateService.initBookingDate(BookConstant.RENT_CLOTHES, 3);
        BookingDate bookingDateOwn = bookingDateService.initBookingDate(BookConstant.OWN_CLOTHES, 3);
        bookingDateList.add(bookingDateRent);
        bookingDateList.add(bookingDateOwn);
        boolean result = bookingDateService.saveBatch(bookingDateList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
    }




    /**
     * 根据预约类别查询未来四天(包括今天)的预约日期
     * @param commonStringRequest  预约类型
     * @return  当前预约类型未来四天的预约日期列表
     */
    @PostMapping("/list")
    @Operation(summary = "(小程序端)根据预约类别查询未来四天(包括今天)的预约日期", description = "参数：预约类型, 权限：所有人，方法名：listBookingDateByType")
    public BaseResponse<List<BookingDateVO>> listBookingDateByType(@RequestBody CommonStringRequest commonStringRequest) {
        if (commonStringRequest == null || StringUtils.isBlank(commonStringRequest.getType())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String type = commonStringRequest.getType();
        QueryWrapper<BookingDate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", type);
        List<BookingDate> bookingDateList = bookingDateService.list(queryWrapper);
        // 封装预约日期VO
        List<BookingDateVO> bookingDateVOS = bookingDateList.stream().map(bookingDate -> {
            BookingDateVO bookingDateVO = new BookingDateVO();
            BeanUtils.copyProperties(bookingDate, bookingDateVO);
            return bookingDateVO;
        }).toList();
        return ResultUtils.success(bookingDateVOS);
    }



    /**
     * 更新当天的预约情况
     * @param bookingDateUpdateRequest  预约日期更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员更新当天的预约情况", description = "参数：预约日期更新请求体, 权限：管理员(admin, boss)，方法名：updateBookingDate")
    public BaseResponse<Boolean> updateBookingDate(@RequestBody BookingDateUpdateRequest bookingDateUpdateRequest) {
        if (bookingDateUpdateRequest == null || bookingDateUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        BookingDate bookingDate = new BookingDate();
        BeanUtils.copyProperties(bookingDateUpdateRequest, bookingDate);
        boolean result = bookingDateService.updateById(bookingDate);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }




}
