package com.cultural.heritage.controller.good;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonDelBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.appointment.AppointmentDateAddRequest;
import com.cultural.heritage.model.dto.appointment.AppointmentDateUpdateRequest;
import com.cultural.heritage.model.dto.good.GoodAddRequest;
import com.cultural.heritage.model.dto.good.GoodQueryRequest;
import com.cultural.heritage.model.dto.good.GoodUpdateRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodAddRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodUpdateRequest;
import com.cultural.heritage.model.entity.AppointmentDate;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.vo.appointment.AppointmentDateVO;
import com.cultural.heritage.model.vo.good.GoodPageVO;
import com.cultural.heritage.model.vo.good.ServiceGoodCardVO;
import com.cultural.heritage.model.vo.good.ServiceGoodVO;
import com.cultural.heritage.service.good.AppointmentDateService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.utils.AppointmentDateUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * 商品接口
 */
@RestController
@RequestMapping("/goods")
@Slf4j
@Tag(name = "商品管理模块")
public class GoodController {


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;


    @Resource
    private AppointmentDateService appointmentDateService;


    /**
     * Web端管理员添加常规类商品
     * @param goodAddRequest  商品添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加常规类商品", description = "参数：商品添加请求体，权限：管理员(admin, boss)，方法名：addGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addGood(@RequestBody GoodAddRequest goodAddRequest) {
        if (goodAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Good good = new Good();
        BeanUtils.copyProperties(goodAddRequest, good);
        // 校验
        goodService.validGood(good, false);
        boolean save = goodService.save(good);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "商品插入成功");
    }




    /**
     * Web端管理员添加服务类商品
     * @param serviceGoodAddRequest  服务类商品添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add/service")
    @Operation(summary = "Web端管理员添加服务类商品", description = "参数：服务类商品添加请求体，权限：管理员(admin, boss)，方法名：addServiceGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addServiceGood(@RequestBody ServiceGoodAddRequest serviceGoodAddRequest) {
        if (serviceGoodAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 向商品表插入服务类商品的基本信息
        Good good = new Good();
        BeanUtils.copyProperties(serviceGoodAddRequest, good);
        // 校验
        goodService.validGood(good, false);
        boolean save = goodService.save(good);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        // 获取当天及未来三天的日期
        List<String> dates = AppointmentDateUtil.getDates();
        AtomicInteger idx = new AtomicInteger(0);

        // 向预约日期表中批量插入服务类商品的预约信息
        Long id = good.getId();
        List<AppointmentDateAddRequest> appointmentDateAddRequestList = serviceGoodAddRequest.getAppointmentDateAddRequestList();
        List<AppointmentDate> appointmentDates = appointmentDateAddRequestList.stream().map(appointmentDateAddRequest -> {
            AppointmentDate appointmentDate = new AppointmentDate();
            BeanUtils.copyProperties(appointmentDateAddRequest, appointmentDate);
            appointmentDate.setSpecificDate(dates.get(idx.getAndIncrement()));
            appointmentDate.setGoodId(id);
            // 校验
            appointmentDateService.validAppointmentDate(appointmentDate, false);
            return appointmentDate;
        }).toList();
        boolean isSuccess = appointmentDateService.saveBatch(appointmentDates);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }



    /**
     * Web端管理员删除常规类商品
     * @param deleteRequest   商品删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除常规类商品", description = "参数：商品删除请求体，权限：管理员(admin, boss)，方法名：deleteGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteGood(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteRequest.getId();
        boolean result = goodService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员删除服务类商品
     * @param deleteRequest     服务类商品删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete/service")
    @Operation(summary = "Web端管理员删除服务类商品", description = "参数: 商品删除请求体，权限：管理员(admin, boss)，方法名：deleteServiceGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> deleteServiceGood(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteRequest.getId();
        // 删除预约日期表中与该商品关联的记录
        QueryWrapper<AppointmentDate> dateQueryWrapper = new QueryWrapper<>();
        dateQueryWrapper.eq("goodId", id);
        boolean isSuccess = appointmentDateService.remove(dateQueryWrapper);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR);

        // 删除商品表中的服务类商品
        boolean result = goodService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }





    /**
     * Web端用户批量删除服务类商品
     * @param ids   服务类商品id列表
     * @return   是否删除成功
     */
    @PostMapping("/delBatch/service")
    @Operation(summary = "Web端用户批量删除服务类商品", description = "服务类商品id列表，权限：管理员(admin, boss)，方法名：delBatchServiceGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> delBatchServiceGoods(@RequestBody List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 删除预约日期表中与该商品关联的记录
        QueryWrapper<AppointmentDate> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("goodId", ids);
        boolean remove = appointmentDateService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "服务类商品预约日期删除失败");

        // 删除商品表中的服务类商品
        boolean result = goodService.removeBatchByIds(ids);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品删除失败");

        return ResultUtils.success(true);

    }



    /**
     * Web端管理员更新常规类商品
     * @param goodUpdateRequest  商品更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新常规类商品", description = "参数：商品更新请求体，权限：管理员(admin, boss)，方法名：updateGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateGoods(@RequestBody GoodUpdateRequest goodUpdateRequest) {
        if (goodUpdateRequest == null || goodUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Good good = new Good();
        BeanUtils.copyProperties(goodUpdateRequest, good);
        // 校验
        goodService.validGood(good, true);
        boolean result = goodService.updateById(good);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }




    /**
     * Web端管理员分页查询常规类商品
     * @param goodQueryRequest  查询商品请求体
     * @return  商品信息列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询常规类商品", description = "参数：商品查询请求体，排序字段 sortField: price inventory id，权限：管理员(admin, boss)，方法名：listGoodByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<GoodPageVO>> listGoodByPage(@RequestBody GoodQueryRequest goodQueryRequest) {
        long current = goodQueryRequest.getCurrent();
        long pageSize = goodQueryRequest.getPageSize();
        QueryWrapper<Good> goodQueryWrapper = goodService.getGoodQueryWrapper(goodQueryRequest);
        Page<Good> page = goodService.page(new Page<>(current, pageSize), goodQueryWrapper);
        List<Good> records = page.getRecords();
        List<GoodPageVO> goodPageVOS = records.stream().map(good -> {
            GoodPageVO goodPageVO = new GoodPageVO();
            BeanUtils.copyProperties(good, goodPageVO);
            return goodPageVO;
        }).toList();
        Page<GoodPageVO> goodPageVOPage = new Page<>();
        goodPageVOPage.setRecords(goodPageVOS);
        goodPageVOPage.setTotal(page.getTotal());
        goodPageVOPage.setSize(page.getSize());
        goodPageVOPage.setCurrent(page.getCurrent());
        goodPageVOPage.setPages(page.getPages());
        return ResultUtils.success(goodPageVOPage);
    }





    /**
     * Web端管理员批量删除常规类商品
     * @param commonDelBatchRequest 批量删除请求体
     * @return 是否批量删除成功
     */
    @PostMapping("/delBatch")
    @Operation(summary = "Web端管理员批量删除常规类商品", description = "参数：商品批量删除请求体，权限：管理员(admin, boss)，方法名：delBatchGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBatchGoods(@RequestBody CommonDelBatchRequest commonDelBatchRequest) {
        List<Long> idList = commonDelBatchRequest.getIdList();
        boolean result = goodService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }





    /**
     * 小程序端用户根据id查询常规类商品
     * @param getByIdRequest  根据id查询请求体
     * @return  商品信息
     */
    @PostMapping("/getById")
    @Operation(summary = "小程序端用户根据id查询常规类商品", description = "参数：商品根据id查询请求体，权限：所有人, 方法名:getGoodById")
    public BaseResponse<GoodPageVO> getGoodById(@RequestBody CommonRequest getByIdRequest) {
        if (getByIdRequest == null || getByIdRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = getByIdRequest.getId();
        Good good = goodService.getById(id);
        ThrowUtils.throwIf(good == null, ErrorCode.NOT_FOUND_ERROR);
        GoodPageVO goodPageVO = new GoodPageVO();
        BeanUtils.copyProperties(good, goodPageVO);
        return ResultUtils.success(goodPageVO);
    }



    /**
     * Web端管理员分页查询服务类商品
     * @param goodQueryRequest  商品查询请求体
     * @return  服务类商品列表
     */
    @PostMapping("/service/list/page")
    @Operation(summary = "Web端管理员分页查询服务类商品", description = "参数：商品查询请求体，权限：管理员(admin, boss)，方法名：listServiceGoodVOByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<ServiceGoodVO>> listServiceGoodVOByPage(@RequestBody GoodQueryRequest goodQueryRequest) {
        if (goodQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = goodQueryRequest.getCurrent();
        long pageSize = goodQueryRequest.getPageSize();
        QueryWrapper<Good> goodQueryWrapper = goodService.getGoodQueryWrapper(goodQueryRequest);
        Page<Good> page = goodService.page(new Page<>(current, pageSize), goodQueryWrapper);
        List<Good> records = page.getRecords();


        // 获取当天及未来三天的日期
        List<String> dates = AppointmentDateUtil.getDates();

        // 用map集合存储服务类商品关联的预约日期
        List<AppointmentDate> appointmentDateList = appointmentDateService.list();
        List<AppointmentDateVO> appointmentDateVOS = appointmentDateList.stream().map(appointmentDate -> {
            AppointmentDateVO appointmentDateVO = new AppointmentDateVO();
            BeanUtils.copyProperties(appointmentDate, appointmentDateVO);
            return appointmentDateVO;
        }).toList();
        Map<Long, List<AppointmentDateVO>> map = new HashMap<>();
        for (AppointmentDateVO appointmentDateVO : appointmentDateVOS) {
            Long goodId = appointmentDateVO.getGoodId();
            String specificDate = appointmentDateVO.getSpecificDate();
            if (!dates.contains(specificDate)) continue;
            List<AppointmentDateVO> dateVOList = map.get(goodId) != null ? map.get(goodId) : new ArrayList<>();
            dateVOList.add(appointmentDateVO);
            map.put(goodId, dateVOList);
        }

        List<ServiceGoodVO> serviceGoodVOList = records.stream().map(good -> {
            ServiceGoodVO serviceGoodVO = new ServiceGoodVO();
            BeanUtils.copyProperties(good, serviceGoodVO);
            Long id = good.getId();
            List<AppointmentDateVO> appointmentDateVOList = map.get(id);
            serviceGoodVO.setAppointmentDateVOS(appointmentDateVOList);
            return serviceGoodVO;
        }).toList();
        Page<ServiceGoodVO> serviceGoodVOPage = new Page<>();
        serviceGoodVOPage.setRecords(serviceGoodVOList);
        serviceGoodVOPage.setCurrent(current);
        serviceGoodVOPage.setPages(pageSize);
        serviceGoodVOPage.setTotal(page.getTotal());
        return ResultUtils.success(serviceGoodVOPage);
    }




    /**
     * 小程序端用户根据id查询服务类商品
     * @param commonRequest 根据id查询请求体
     * @return 服务类商品信息
     */
    @PostMapping("/service/list/id")
    @Operation(summary = "小程序端用户根据id查询服务类商品", description = "参数：根据id查询请求体，权限：所有人，方法名：listServiceGoodVO")
    public BaseResponse<ServiceGoodVO> listServiceGoodVO(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        Good good = goodService.getById(id);
        ThrowUtils.throwIf(good == null, ErrorCode.NOT_FOUND_ERROR, "商品不存在");
        ServiceGoodVO serviceGoodVO = new ServiceGoodVO();
        BeanUtils.copyProperties(good, serviceGoodVO);

        // 获取当天及未来三天的日期
        List<String> dates = AppointmentDateUtil.getDates();

        // 获取当前服务类商品关联的预约日期
        QueryWrapper<AppointmentDate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("goodId", id);
        List<AppointmentDate> appointmentDateList = appointmentDateService.list(queryWrapper);
        List<AppointmentDateVO> appointmentDateVOS = appointmentDateList.stream().map(appointmentDate -> {
            AppointmentDateVO appointmentDateVO = new AppointmentDateVO();
            BeanUtils.copyProperties(appointmentDate, appointmentDateVO);
            return appointmentDateVO;
        }).toList();
        List<AppointmentDateVO> dateVOList = appointmentDateVOS.stream()
                .filter(appointmentDateVO -> dates.contains(appointmentDateVO.getSpecificDate())).toList();
        serviceGoodVO.setAppointmentDateVOS(dateVOList);
        return ResultUtils.success(serviceGoodVO);
    }



    /**
     * 小程序端展示服务类商品卡片
     * @return  服务类商品卡片列表
     */
    @GetMapping("/service/list/card")
    @Operation(summary = "小程序端展示服务类商品卡片", description = "参数：无，权限：所有人，方法名：listServiceGoodCardVO")
    public BaseResponse<List<ServiceGoodCardVO>> listServiceGoodCardVO(HttpServletRequest request) {
        userService.getLoginUser(request);
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("isGoodType", 0);
        List<Good> goodList = goodService.list(queryWrapper);
        List<ServiceGoodCardVO> serviceGoodCardVOS = goodList.stream().map(good -> {
            ServiceGoodCardVO serviceGoodCardVO = new ServiceGoodCardVO();
            BeanUtils.copyProperties(good, serviceGoodCardVO);
            return serviceGoodCardVO;
        }).toList();
        return ResultUtils.success(serviceGoodCardVOS);
    }





    /**
     * Web端管理员更新服务类商品
     * @param serviceGoodUpdateRequest  服务类商品更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/service/update")
    @Operation(summary = "Web端管理员更新服务类商品", description = "参数：服务类商品更新请求体，权限：管理员(admin, boss)，方法名：updateServiceGoodById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateServiceGoodById(@RequestBody ServiceGoodUpdateRequest serviceGoodUpdateRequest) {
        if (serviceGoodUpdateRequest == null || serviceGoodUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新服务类商品的基本信息
        Good good = new Good();
        BeanUtils.copyProperties(serviceGoodUpdateRequest, good);
        good.setIsGoodType(0);
        // 校验
        goodService.validGood(good, true);
        boolean result = goodService.updateById(good);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "商品不存在");

        // 获取当天及未来三天的日期
        List<String> dates = AppointmentDateUtil.getDates();
        AtomicInteger idx = new AtomicInteger(0);

        // 封装预约日期实体类
        List<AppointmentDateUpdateRequest> appointmentDateUpdateRequestList = serviceGoodUpdateRequest.getAppointmentDateUpdateRequestList();
        List<AppointmentDate> appointmentDateList = appointmentDateUpdateRequestList.stream().map(appointmentDateUpdateRequest -> {
            AppointmentDate appointmentDate = new AppointmentDate();
            BeanUtils.copyProperties(appointmentDateUpdateRequest, appointmentDate);
            appointmentDate.setSpecificDate(dates.get(idx.getAndIncrement()));
            appointmentDate.setGoodId(good.getId());
            // 校验
            appointmentDateService.validAppointmentDate(appointmentDate, true);
            return appointmentDate;
        }).toList();

        // 更新当前服务类商品的预约日期
        boolean isSuccess = appointmentDateService.updateBatchById(appointmentDateList);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }




    /**
     * (服务类商品)初始化未来一天的预约日期
     */
//        @Scheduled(cron = "*/5 * * * * ?")
    // 每天00:00-00:00调用
    @Scheduled(cron = "0 0 0 * * ?")
    @PostMapping("/service/init")
    @Operation(summary = "(服务类商品)初始化未来一天的预约日期", description = "参数：无，权限：自动调用，方法名：initServiceGoodBookingDate")
    @Transactional(rollbackFor = Exception.class)
    public void initServiceGoodBookingDate() {
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("isGoodType", 0);
        List<Good> goodList = goodService.list(queryWrapper);
        List<AppointmentDate> appointmentDateList = new ArrayList<>();
        goodList.forEach(good -> {
            AppointmentDate appointmentDate = new AppointmentDate();
            appointmentDate.setGoodId(good.getId());
            appointmentDate.setSpecificDate(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), 3)));
            appointmentDate.setTimeSlot("00:00-00:00");
            appointmentDate.setIsAvailable(0);
            appointmentDate.setNumberRange("(0,0)");
            appointmentDateList.add(appointmentDate);
        });
        boolean result = appointmentDateService.saveBatch(appointmentDateList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "初始化失败");
    }





}
